/* gnubiff -- a mail notification program
 * Copyright (c) 2000-2004 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <sstream>
#include <cstdio>
#include <string>
#include <glib.h>
#include "Applet.h"
#include "Mailbox.h"
#include "Popup.h"


// ================================================================================
//  Constructors & Destructors
// --------------------------------------------------------------------------------
//  
// ================================================================================
Applet::Applet (Biff *owner, std::string xmlFilename) : GUI (xmlFilename)
{
	_owner = owner;
	_popup = false;
	_process_mutex = g_mutex_new ();
	_update_mutex = g_mutex_new ();
}

Applet::~Applet (void)
{
	g_mutex_lock (_process_mutex);
	g_mutex_unlock (_process_mutex);
	g_mutex_free (_process_mutex);
	g_mutex_lock (_update_mutex);
	g_mutex_unlock (_update_mutex);
	g_mutex_free (_update_mutex);
}


// ================================================================================
//  Watch functions
// --------------------------------------------------------------------------------
//  Those function handles automatic and forced watch
// ================================================================================
void Applet::watch (void)
{
	_popup = true;
	for (unsigned int i=0; i<_owner->mailboxes(); i++)
		_owner->mailbox(i)->watch();
}

void Applet::watch_now (void)
{
	_popup = false;
	for (unsigned int i=0; i<_owner->mailboxes(); i++)
		_owner->mailbox(i)->watch();
}

void Applet::watch_on (void)
{
	for (unsigned int i=0; i<_owner->mailboxes(); i++)
		_owner->mailbox(i)->watch_on();
}

void Applet::watch_off (void)
{
	for (unsigned int i=0; i<_owner->mailboxes(); i++)
		_owner->mailbox(i)->watch_off();
}


// ================================================================================
//  Process mailbox
// --------------------------------------------------------------------------------
//  This function decide what to do with current mailboxes states
// ================================================================================
void Applet::process (void) {
	// Are we already updating applet ?
	if (!g_mutex_trylock (_process_mutex))
		return;
  
	// Look for status of mailboxes
	gboolean newmail = false;
	int unread = 0;
	for (unsigned int i=0; i<_owner->mailboxes(); i++) {
		if (_owner->mailbox(i)->status() == MAILBOX_NEW)
			newmail = true;
		unread += _owner->mailbox(i)->unreads();
	}

	// We play sound only if watch was not asked explicitely
	if ((newmail == true) && (unread > 0) && (_popup == false)) {
		if (_owner->_sound_type == SOUND_BEEP) {
			gdk_beep ();    
		}
		else if (_owner->_sound_type == SOUND_FILE) {
			std::stringstream s;
			s << _owner->_sound_volume/100.0;
			std::string command = "play \"" + _owner->_sound_file + std::string ("\" -v ");
			command += s.str();
			command += " &";
			system (command.c_str());
		}
	}

	// Check if we display popup window depending on popup value:
	if (unread && ((_owner->_display_popup && newmail) || (_popup))) {
		_owner->popup()->update();
		_owner->popup()->show();
	}
	else {
		watch_on();
	}

	_popup = false;
	g_mutex_unlock (_process_mutex);
}
