/* gnubiff -- a mail notification program
 * Copyright (C) 2000-2004 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <sstream>
#include <cstdio>
#include "AppletGNOME.h"
#include "Mailbox.h"
#include "Popup.h"



// ===================================================================
// = Callbacks "C" binding ===========================================
// ===================================================================
extern "C" {
	void APPLET_GNOME_on_enter (GtkWidget *widget, GdkEventCrossing *event, gpointer data)
	{((AppletGNOME *) data)->tooltip_update ();}
	gboolean APPLET_GNOME_on_change_orient (GtkWidget *widget, PanelAppletOrient orient, gpointer data)
	{return ((AppletGNOME *) data)->on_change_orient ();}
	gboolean APPLET_GNOME_on_change_size (GtkWidget *widget, int size, gpointer data)
	{return ((AppletGNOME *) data)->on_change_size ();}
	gboolean APPLET_GNOME_on_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
	{return ((AppletGNOME *) data)->on_button_press (event);}
	void APPLET_GNOME_on_menu_properties (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
	{((AppletGNOME *) data)->on_menu_properties (uic, verbname);}
	void APPLET_GNOME_on_menu_mail_app (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
	{((AppletGNOME *) data)->on_menu_mail_app (uic, verbname);}
	void APPLET_GNOME_on_menu_mail_read (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
	{((AppletGNOME *) data)->on_menu_mail_read (uic, verbname);}
	void APPLET_GNOME_on_menu_about (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
	{((AppletGNOME *) data)->on_menu_about (uic, verbname);}
}


// ================================================================================
//  Constructors & Destructors
// --------------------------------------------------------------------------------
//  
// ================================================================================
AppletGNOME::AppletGNOME (Biff *owner, std::string xmlFilename) : Applet (owner, xmlFilename)
{
	_about = 0;
}

AppletGNOME::~AppletGNOME (void)
{
}


// ================================================================================
//  Docking
// --------------------------------------------------------------------------------
//  
// ================================================================================
void AppletGNOME::dock (GtkWidget *applet) {
	static const BonoboUIVerb gnubiffMenuVerbs [] = {
		BONOBO_UI_VERB ("Props",   APPLET_GNOME_on_menu_properties),
		BONOBO_UI_VERB ("MailApp", APPLET_GNOME_on_menu_mail_app),
		BONOBO_UI_VERB ("MailRead", APPLET_GNOME_on_menu_mail_read),
		BONOBO_UI_VERB ("About",   APPLET_GNOME_on_menu_about), 
		BONOBO_UI_VERB_END
	};
 
	panel_applet_setup_menu_from_file (PANEL_APPLET (applet),
									   NULL,
									   "GNOME_gnubiffApplet.xml",
									   NULL,
									   gnubiffMenuVerbs,
									   this);
	gtk_widget_reparent (get ("ebox"), applet);  
	g_signal_connect (G_OBJECT (applet),"change_orient",       GTK_SIGNAL_FUNC (APPLET_GNOME_on_change_orient), this);
	g_signal_connect (G_OBJECT (applet),"change_size",         GTK_SIGNAL_FUNC (APPLET_GNOME_on_change_size), this);
	g_signal_connect (G_OBJECT (applet), "button_press_event", GTK_SIGNAL_FUNC (APPLET_GNOME_on_button_press), this);

	gtk_widget_show (applet);

	_gnome_applet = applet;

	return;
}

// ===================================================================
// = update ==========================================================
/**
   Update applet (title, image, labels)
*/
// ===================================================================
void AppletGNOME::update (void) {
	// Is there another update going on ?
	if (!g_mutex_trylock (_update_mutex))
		return;

	// Applet label (number of mail)
	std::string text;
	text = "<span font_desc=\"";
	text += _owner->_applet_font;
	text += "\">";
	text += "<span color=\"";
	text += _owner->_applet_font_color;
	text += "\"><b>";
	int unread = 0;
	for (unsigned int i=0; i<_owner->mailboxes(); i++)
		unread += _owner->mailbox(i)->unreads();
	std::stringstream unreads;
	unreads << unread;
	if (unread < (int)(_owner->_max_collected_mail))
		text += unreads.str();
	else
		text += "++";
	text += "</b></span></span>";

	gtk_label_set_markup (GTK_LABEL(get ("hunread")), text.c_str());
	gtk_label_set_markup (GTK_LABEL(get ("vunread")), text.c_str());

	GdkPixbufAnimation *gdkpix;
	if (unread > 0)
		gdkpix = gtk_image_get_animation (GTK_IMAGE(_owner->setup()->get("mail_image_image")));
	else
		gdkpix = gtk_image_get_animation (GTK_IMAGE(_owner->setup()->get("no_mail_image_image")));

	gtk_image_set_from_animation (GTK_IMAGE (get("image")), gdkpix);


	guint size = panel_applet_get_size (PANEL_APPLET (_gnome_applet));
	PanelAppletOrient orient = panel_applet_get_orient (PANEL_APPLET (_gnome_applet));
	guint width = gdk_pixbuf_animation_get_width (gdkpix);
	guint height = gdk_pixbuf_animation_get_height (gdkpix);
	gboolean too_big = false;

	// The panel is oriented horizontally
	if ((orient == PANEL_APPLET_ORIENT_DOWN) || (orient == PANEL_APPLET_ORIENT_UP)) {
		gtk_widget_hide (get ("vunread"));
		gtk_widget_show (get ("hunread"));
		if (height > size) {
			gtk_widget_hide (get ("image"));
			too_big = true;
		}
	}
	// The panel is oriented vertically
	else {
		gtk_widget_hide (get ("hunread"));
		gtk_widget_show (get ("vunread"));
		if (width > size)
			gtk_widget_hide (get ("image"));
	}

	g_mutex_unlock (_update_mutex);
}


// ================================================================================
//  Show/Hide
// --------------------------------------------------------------------------------
//  
// ================================================================================
void AppletGNOME::show (std::string name)
{
	gtk_widget_show (_gnome_applet);
}

void AppletGNOME::hide (std::string name)
{
	gtk_widget_hide (_gnome_applet);
}


// ================================================================================
//  Various callbacks
// --------------------------------------------------------------------------------
// 
// ================================================================================
void AppletGNOME::tooltip_update (void)
{
	std::string tooltip;
	for (unsigned int i=0; i<_owner->mailboxes(); i++) {
		tooltip += _owner->mailbox(i)->name();
		tooltip += " : ";
		std::stringstream s;
		s << _owner->mailbox(i)->unreads();
		if (_owner->mailbox(i)->status() == MAILBOX_ERROR)
			tooltip += _("error !");
		else if (_owner->mailbox(i)->status() == MAILBOX_CHECKING) {
			tooltip += "(";
			tooltip += s.str();
			tooltip += ")";
			tooltip += _(" checking...");
		}
		else
			tooltip += s.str();

		if (i< (_owner->mailboxes()-1))
			tooltip += "\n";
	}
	GtkTooltipsData *tt = gtk_tooltips_data_get  (get("ebox"));
	gtk_tooltips_set_tip (tt->tooltips, get("ebox"), tooltip.c_str(), "");
}

gboolean AppletGNOME::on_change_orient (void)
{
	update();
	return TRUE;
}

gboolean AppletGNOME::on_change_size (void)
{
	update();
	return TRUE;
}

gboolean AppletGNOME::on_button_press (GdkEventButton *event)
{
	// Double left click : start mail app
	if ((event->type == GDK_2BUTTON_PRESS) && (event->button == 1)) {
		std::string command = _owner->_mail_app + " &";
		system (command.c_str());
	}
	// Single left click : popup menu
	else if (event->button == 1)
		watch();
	// Single middle click : mark mails as read
	else if (event->button == 2) {
		for (unsigned int i=0; i<_owner->mailboxes(); i++)
			_owner->mailbox(i)->mark_all();
		_popup = true;
		_owner->popup()->hide();
		_owner->applet()->process();
		_owner->applet()->update();
	}

	return FALSE;
}

void AppletGNOME::on_menu_properties (BonoboUIComponent *uic, const gchar *verbname)
{
	_owner->applet()->watch_off();
	_owner->popup()->hide();
	_owner->setup()->show();
	if (_about != 0)
		gdk_window_hide (_about->window);
}

void AppletGNOME::on_menu_mail_app (BonoboUIComponent *uic, const gchar *verbname)
{
	std::string command = _owner->_mail_app + " &";
	system (command.c_str());
}

void AppletGNOME::on_menu_mail_read (BonoboUIComponent *uic, const gchar *verbname)
{
	for (unsigned int i=0; i<_owner->mailboxes(); i++)
		_owner->mailbox(i)->mark_all();
	_popup = true;
	_owner->popup()->hide();
	_owner->applet()->process();
	_owner->applet()->update();
}

void AppletGNOME::on_menu_about (BonoboUIComponent *uic, const gchar *verbname)
{
	static const gchar *authors[] = {
		"Nicolas Rougier <Nicolas.Rougier@loria.fr>",
		NULL
	};
 
	if (_about != 0) {
		gdk_window_show (_about->window);
		gdk_window_raise (_about->window);
		return;
	}
	_about = gnome_about_new ("gnubiff applet", PACKAGE_VERSION,
							  "(C) 2000-2004\n ",
							  _("Released under the GNU general public license.\n"\
								"Mail notification program."),
							  authors,
							  NULL, NULL, gdk_pixbuf_new_from_file (GNUBIFF_DATADIR"/logo.png", 0));
	g_signal_connect (G_OBJECT (_about), "destroy", G_CALLBACK (gtk_widget_destroyed), &_about);
	gtk_widget_show (_about);
}
