/* gnubiff -- a mail notification program
 * Copyright (c) 2000-2004 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <sstream>
#include <cstdio>
#include <string>
#include <glib.h>
#include "AppletGTK.h"
#include "Mailbox.h"
#include "Popup.h"


// ================================================================================
//  Callbacks (C binding)
// ================================================================================
extern "C" {
	void APPLET_GTK_on_enter (GtkWidget *widget, GdkEventCrossing *event, gpointer data)
	{((AppletGTK *) data)->tooltip_update ();}
	gboolean APPLET_GTK_on_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
	{return ((AppletGTK *) data)->on_button_press (event);}
	void APPLET_GTK_on_menu_mail_app (GtkWidget *widget, gpointer data)
	{((AppletGTK *) data)->on_menu_mail_app ();}
	void APPLET_GTK_on_menu_mark (GtkWidget *widget, gpointer data)
	{((AppletGTK *) data)->on_menu_mark ();}
	void APPLET_GTK_on_menu_preferences (GtkWidget *widget, gpointer data)
	{((AppletGTK *) data)->on_menu_preferences ();}
	void APPLET_GTK_on_menu_about (GtkWidget *widget, gpointer data)
	{((AppletGTK *) data)->on_menu_about ();}
	void APPLET_GTK_on_hide_about (GtkWidget *widget, gpointer data)
	{((AppletGTK *) data)->on_hide_about ();}
}



// ================================================================================
//  Constructors & Destructors
// --------------------------------------------------------------------------------
//  
// ================================================================================
AppletGTK::AppletGTK (Biff *owner, std::string xmlFilename) : Applet (owner, xmlFilename)
{
}

AppletGTK::~AppletGTK (void)
{
}


// ================================================================================
//  Update applet according to mailboxes state
// --------------------------------------------------------------------------------
//  
// ================================================================================
void AppletGTK::update (void) {
	// Is there another update going on ?
	if (!g_mutex_trylock (_update_mutex))
		return;

	// Applet label (number of mail)
	std::string text;
	text = "<span font_desc=\"";
	text += _owner->_applet_font;
	text += "\">";
	text += "<span color=\"";
	text += _owner->_applet_font_color;
	text += "\"><b>";
	int unread = 0;
	for (unsigned int i=0; i<_owner->mailboxes(); i++)
		unread += _owner->mailbox(i)->unreads();
	std::stringstream unreads;
	unreads << unread;
	if (unread < (int)(_owner->_max_collected_mail))
		text += unreads.str();
	else
		text += "++";
	text += "</b></span></span>";

	// Image/animation
	GdkPixbufAnimation *gdkpix;
	if (unread > 0)
		gdkpix = gtk_image_get_animation (GTK_IMAGE(_owner->setup()->get("mail_image_image")));
	else
		gdkpix = gtk_image_get_animation (GTK_IMAGE(_owner->setup()->get("no_mail_image_image")));

	gtk_label_set_markup (GTK_LABEL(get ("unread")), text.c_str());

	gtk_image_set_from_animation (GTK_IMAGE (get("image")), gdkpix);

	
	// Update widgets relative to image size
	guint width = gdk_pixbuf_animation_get_width (gdkpix);
	guint height = gdk_pixbuf_animation_get_height (gdkpix);
	gtk_widget_set_size_request (get("fixed"), width, height);
	gtk_widget_set_size_request (get("image"), width, height);
	gtk_widget_set_size_request (get("unread"), width, -1);

	GtkRequisition req;
	gtk_widget_size_request (get("unread"), &req);
	gtk_fixed_move (GTK_FIXED(get("fixed")), get("unread"), 0, height-req.height);

	// Update window manager decorations
	gtk_window_set_decorated (GTK_WINDOW(get("dialog")), _owner->_biff_decorated);
	tooltip_update();
	show();

	g_mutex_unlock (_update_mutex);
}


// ================================================================================
//  Show the applet
// --------------------------------------------------------------------------------
//  Show the applet (at the right place if necessary)
// ================================================================================
void AppletGTK::show (std::string name)
{
	std::string geometry  = "";
	if (_owner->_biff_x_sign == SIGN_PLUS)
		geometry += "+";
	else
		geometry += "-";
	std::stringstream x;
	x << _owner->_biff_x;
	geometry += x.str();

	if (_owner->_biff_y_sign == SIGN_PLUS)
		geometry += "+";
	else
		geometry += "-";
	std::stringstream y;
	y << _owner->_biff_y;
	geometry += y.str();

	gtk_widget_show (get("dialog"));
	if (!_owner->_biff_no_geometry)
		gtk_window_parse_geometry (GTK_WINDOW(get("dialog")), geometry.c_str());
}


// ================================================================================
//  Various callbacks
// --------------------------------------------------------------------------------
// 
// ================================================================================
void AppletGTK::tooltip_update (void)
{
	std::string tooltip;
	for (unsigned int i=0; i<_owner->mailboxes(); i++) {
		tooltip += _owner->mailbox(i)->name();
		tooltip += " : ";
		std::stringstream s;
		s << _owner->mailbox(i)->unreads();
		if (_owner->mailbox(i)->status() == MAILBOX_ERROR)
			tooltip += _("error !");
		else if (_owner->mailbox(i)->status() == MAILBOX_CHECKING) {
			tooltip += "(";
			tooltip += s.str();
			tooltip += ")";
			tooltip += _(" checking...");
		}
		else
			tooltip += s.str();

		if (i< (_owner->mailboxes()-1))
			tooltip += "\n";
	}
	GtkTooltipsData *tt = gtk_tooltips_data_get  (get("dialog"));
	gtk_tooltips_set_tip (tt->tooltips, get("dialog"), tooltip.c_str(), "");
}

gboolean AppletGTK::on_button_press (GdkEventButton *event)
{

	// Double left click : start mail app
	if ((event->type == GDK_2BUTTON_PRESS) && (event->button == 1))
		on_menu_mail_app ();

	// Single left click : force mail check
	else if (event->button == 1)
		watch();

	// Single middle click : mark mails as read
	else if (event->button == 2) {
		for (unsigned int i=0; i<_owner->mailboxes(); i++)
			_owner->mailbox(i)->mark_all();
		_popup = true;
		_owner->popup()->hide();
		_owner->applet()->process();
		_owner->applet()->update();
	}

	// Single right click : popup menu
	else if (event->button == 3)
		gtk_menu_popup (GTK_MENU(get("menu")), NULL, NULL, NULL, NULL, event->button, event->time);
	return true;
}

// ================================================================================
void AppletGTK::on_menu_mail_app (void)
{
	std::string command = _owner->_mail_app + " &";
	system (command.c_str());	
}

// ================================================================================
void AppletGTK::on_menu_preferences (void)
{
	_owner->applet()->watch_off();
	_owner->popup()->hide();
	_owner->setup()->show();
}

// ================================================================================
void AppletGTK::on_menu_mark (void)
{
	for (unsigned int i=0; i<_owner->mailboxes(); i++)
		_owner->mailbox(i)->mark_all();
	_popup = true;
	_owner->applet()->process();
	_owner->applet()->update();
}

// ================================================================================
void AppletGTK::on_menu_about (void)
{
	std::string package = "<span size=\"xx-large\"><b>";
	package += PACKAGE_STRING;
	package += "</b></span>";
	gtk_label_set_markup (GTK_LABEL (get("about_version")), package.c_str());
	GUI::show ("about");
}

// ================================================================================
void AppletGTK::on_hide_about (void)
{
	GUI::hide ("about");
}
